import * as i0 from '@angular/core';
import { EventEmitter, Directive, HostBinding, Output, Pipe, Component, HostListener, NgModule } from '@angular/core';
import { Subject, take, tap, concatMap, combineLatest, startWith, takeUntil, map } from 'rxjs';
import { trigger, state, style, transition, animate } from '@angular/animations';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i2 from '@angular/material/core';
import { MatRippleModule } from '@angular/material/core';
import { ComponentPortal } from '@angular/cdk/portal';
import * as i1$1 from '@angular/cdk/overlay';
import { OverlayModule } from '@angular/cdk/overlay';

class AngularMaterialTimepickerBase {
    constructor() {
        /** Defines the default class for the component */
        this.class = "amt";
        /** Notifier for the click on an hour */
        this.hourClicked = new EventEmitter();
        /** Notifier for the click on a minute */
        this.minuteClicked = new EventEmitter();
        /** Hours array that will be used to build the html nodes */
        this.hours = [];
        /** Minutes array that will be used to build the html nodes */
        this.minutes = [];
        /** Current state of the panel animation. */
        this._panelAnimationState = 'void';
        /** Emits whenever an animation on the menu completes. */
        this._animationDone = new Subject();
    }
    /** Callback that is invoked when the panel animation completes. */
    _onAnimationDone(event) {
        this._animationDone.next(event);
        this._isAnimating = false;
    }
    /** Starts the enter animation. */
    _startAnimation() {
        this._panelAnimationState = 'enter';
    }
    /** Resets the panel animation to its initial state. */
    _resetAnimation() {
        this._panelAnimationState = 'void';
    }
}
AngularMaterialTimepickerBase.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerBase, deps: [], target: i0.ɵɵFactoryTarget.Directive });
AngularMaterialTimepickerBase.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.9", type: AngularMaterialTimepickerBase, outputs: { minuteClicked: "minuteClicked" }, host: { properties: { "class": "this.class" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerBase, decorators: [{
            type: Directive
        }], propDecorators: { class: [{
                type: HostBinding,
                args: ['class']
            }], minuteClicked: [{
                type: Output
            }] } });

/** This piece of code is extracted from the Angular Material MatMenu Github */
/**
 * Animations used by the angular-material-timepicker component.
 * Animation duration and timing values are based on:
 * https://material.io/guidelines/components/menus.html#menus-usage
 */
const timepickerAnimations = {
    /**
     * This animation controls the timepicker panel's entry and exit from the page.
     *
     * When the timepicker panel is added to the DOM, it scales in and fades in its border.
     *
     * When the timepicker panel is removed from the DOM, it simply fades out after a brief
     * delay to display the ripple.
     */
    transformTimepicker: trigger('transformTimepicker', [
        state('void', style({
            opacity: 0,
            transform: 'scale(0.8)',
        })),
        transition('void => enter', animate('120ms cubic-bezier(0, 0, 0.2, 1)', style({
            opacity: 1,
            transform: 'scale(1)',
        }))),
        transition('* => void', animate('100ms 25ms linear', style({ opacity: 0 }))),
    ]),
    /**
     * This animation fades in the background color and content of the menu panel
     * after its containing element is scaled in.
     */
    fadeInItems: trigger('fadeInItems', [
        state('showing', style({ opacity: 1 })),
        transition('void => *', [
            style({ opacity: 0 }),
            animate('400ms 100ms cubic-bezier(0.55, 0, 0.55, 0.2)'),
        ]),
    ]),
};

class HourFormatPipe {
    transform(value) {
        const _value = value ? value.toString() : "00";
        return _value.length == 1 ? `0${_value}` : _value;
    }
}
HourFormatPipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: HourFormatPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
HourFormatPipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: HourFormatPipe, name: "hourFormat" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: HourFormatPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'hourFormat'
                }]
        }] });

class MinuteFormatPipe {
    transform(value) {
        const _value = value.toString();
        return _value.length == 1 ? `0${_value}` : _value;
    }
}
MinuteFormatPipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: MinuteFormatPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
MinuteFormatPipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: MinuteFormatPipe, name: "minuteFormat" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: MinuteFormatPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'minuteFormat'
                }]
        }] });

class AngularMaterialTimepickerComponent extends AngularMaterialTimepickerBase {
    constructor() {
        super();
        /** Mode for building the hour structure */
        this.mode = 'pm';
        /** The current hour selected by the component to be shown */
        this.hour = "00";
        /** The current hour selected by the component to be shown */
        this.minute = "00";
        this.buildHours();
        const _default = new Date();
        const pipe = new HourFormatPipe();
        this.hour = pipe.transform(_default.getHours());
        this.minute = pipe.transform(_default.getMinutes());
        // We always have 60 minutes 😂;
        this.minutes = Array.from(Array(60).keys());
    }
    ngAfterViewInit() {
        /**
         * Get the current hour / minute div element and do a scroll in order to show them to the user.
         *
         * We do that because if the input field (which fires the component from the directive) has a valid time format (HH:MM) the component is pre-populated with that data. So on that point, if we have a value different of "00" on the hour / minute variable, the code below shows that to the user.
         */
        const currentHourDiv = document.querySelector('.amt-hour.amt-current');
        const currentMinuteDiv = document.querySelector('.amt-minute.amt-current');
        this._animationDone
            .pipe(take(1))
            .subscribe(() => {
            if (currentHourDiv.offsetTop > 0)
                currentHourDiv.parentElement?.scrollTo({
                    top: currentHourDiv.offsetTop,
                    behavior: "smooth"
                });
            if (currentMinuteDiv.offsetTop > 0)
                currentMinuteDiv.parentElement?.scrollTo({
                    top: currentMinuteDiv.offsetTop,
                    behavior: "smooth"
                });
        });
    }
    ngOnChanges(changes) {
        if (changes["mode"]) {
            this.buildHours(changes["mode"].currentValue);
        }
    }
    /** Builds the hour structure taking into account the mode */
    buildHours(mode) {
        const _mode = mode || this.mode;
        this.hours = Array.from(Array(_mode == 'am' ? 12 : 24).keys());
    }
    /** Sets the current hour or minutes once the user did clicked the view */
    elementClicked(type, value) {
        let pipe;
        switch (type) {
            case 'hour':
                pipe = new HourFormatPipe();
                const _hour = pipe.transform(value);
                this.hour = _hour;
                this.hourClicked.emit(_hour);
                break;
            case 'minute':
                pipe = new MinuteFormatPipe();
                const _minute = pipe.transform(value);
                this.minute = _minute;
                this.minuteClicked.emit(_minute);
                break;
            default:
                pipe = new HourFormatPipe();
                this.hourClicked.emit(pipe.transform(value));
                break;
        }
    }
}
AngularMaterialTimepickerComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
AngularMaterialTimepickerComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.9", type: AngularMaterialTimepickerComponent, selector: "angular-material-timepicker", usesInheritance: true, usesOnChanges: true, ngImport: i0, template: "<div\n  class=\"amt-container\"\n  [@transformTimepicker]=\"_panelAnimationState\"\n  (@transformTimepicker.done)=\"_onAnimationDone($event)\"\n>\n  <div class=\"amt-hours\">\n    <div\n      matRipple\n      (click)=\"elementClicked('hour', _hour)\"\n      class=\"amt-hour\"\n      [ngClass]=\"{ 'amt-current': (_hour | hourFormat) == hour }\"\n      *ngFor=\"let _hour of hours\"\n      #tplHour\n    >\n      {{ _hour | hourFormat }}\n    </div>\n  </div>\n  <div class=\"amt-minutes\">\n    <div\n      matRipple\n      (click)=\"elementClicked('minute', _minute)\"\n      class=\"amt-minute\"\n      [ngClass]=\"{ 'amt-current': (_minute | minuteFormat) == minute }\"\n      *ngFor=\"let _minute of minutes\"\n    >\n      {{ _minute | minuteFormat }}\n    </div>\n  </div>\n</div>\n", styles: ["::ng-deep angular-material-timepicker{height:250px;width:120px}::ng-deep angular-material-timepicker .amt-container{display:grid;height:100%;width:100%;box-shadow:0 2px 4px -1px #0003,0 4px 5px #00000024,0 1px 10px #0000001f;border-radius:4px;background-color:#fff;grid-template-columns:1fr 1fr;grid-template-rows:1fr;grid-gap:0px;gap:0px;overflow:hidden;-ms-overflow-style:none;scrollbar-width:none}::ng-deep angular-material-timepicker .amt-container ::-webkit-scrollbar{display:none}::ng-deep angular-material-timepicker .amt-container .amt-hours{border-right:1px solid rgba(0,0,0,.12)}::ng-deep angular-material-timepicker .amt-container .amt-hours,::ng-deep angular-material-timepicker .amt-container .amt-minutes{overflow-y:scroll;-webkit-overflow-scrolling:touch}::ng-deep angular-material-timepicker .amt-container .amt-hours .amt-current,::ng-deep angular-material-timepicker .amt-container .amt-minutes .amt-current{background-color:#0000001f}::ng-deep angular-material-timepicker .amt-container .amt-hours .amt-hour,::ng-deep angular-material-timepicker .amt-container .amt-hours .amt-minute,::ng-deep angular-material-timepicker .amt-container .amt-minutes .amt-hour,::ng-deep angular-material-timepicker .amt-container .amt-minutes .amt-minute{display:flex;height:50px;font-size:larger;justify-content:center;align-items:center;cursor:pointer}::ng-deep .amt-overlay{background-color:transparent}\n"], directives: [{ type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i2.MatRipple, selector: "[mat-ripple], [matRipple]", inputs: ["matRippleColor", "matRippleUnbounded", "matRippleCentered", "matRippleRadius", "matRippleAnimation", "matRippleDisabled", "matRippleTrigger"], exportAs: ["matRipple"] }, { type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }], pipes: { "hourFormat": HourFormatPipe, "minuteFormat": MinuteFormatPipe }, animations: [timepickerAnimations.transformTimepicker, timepickerAnimations.fadeInItems] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerComponent, decorators: [{
            type: Component,
            args: [{ selector: 'angular-material-timepicker', animations: [timepickerAnimations.transformTimepicker, timepickerAnimations.fadeInItems], template: "<div\n  class=\"amt-container\"\n  [@transformTimepicker]=\"_panelAnimationState\"\n  (@transformTimepicker.done)=\"_onAnimationDone($event)\"\n>\n  <div class=\"amt-hours\">\n    <div\n      matRipple\n      (click)=\"elementClicked('hour', _hour)\"\n      class=\"amt-hour\"\n      [ngClass]=\"{ 'amt-current': (_hour | hourFormat) == hour }\"\n      *ngFor=\"let _hour of hours\"\n      #tplHour\n    >\n      {{ _hour | hourFormat }}\n    </div>\n  </div>\n  <div class=\"amt-minutes\">\n    <div\n      matRipple\n      (click)=\"elementClicked('minute', _minute)\"\n      class=\"amt-minute\"\n      [ngClass]=\"{ 'amt-current': (_minute | minuteFormat) == minute }\"\n      *ngFor=\"let _minute of minutes\"\n    >\n      {{ _minute | minuteFormat }}\n    </div>\n  </div>\n</div>\n", styles: ["::ng-deep angular-material-timepicker{height:250px;width:120px}::ng-deep angular-material-timepicker .amt-container{display:grid;height:100%;width:100%;box-shadow:0 2px 4px -1px #0003,0 4px 5px #00000024,0 1px 10px #0000001f;border-radius:4px;background-color:#fff;grid-template-columns:1fr 1fr;grid-template-rows:1fr;grid-gap:0px;gap:0px;overflow:hidden;-ms-overflow-style:none;scrollbar-width:none}::ng-deep angular-material-timepicker .amt-container ::-webkit-scrollbar{display:none}::ng-deep angular-material-timepicker .amt-container .amt-hours{border-right:1px solid rgba(0,0,0,.12)}::ng-deep angular-material-timepicker .amt-container .amt-hours,::ng-deep angular-material-timepicker .amt-container .amt-minutes{overflow-y:scroll;-webkit-overflow-scrolling:touch}::ng-deep angular-material-timepicker .amt-container .amt-hours .amt-current,::ng-deep angular-material-timepicker .amt-container .amt-minutes .amt-current{background-color:#0000001f}::ng-deep angular-material-timepicker .amt-container .amt-hours .amt-hour,::ng-deep angular-material-timepicker .amt-container .amt-hours .amt-minute,::ng-deep angular-material-timepicker .amt-container .amt-minutes .amt-hour,::ng-deep angular-material-timepicker .amt-container .amt-minutes .amt-minute{display:flex;height:50px;font-size:larger;justify-content:center;align-items:center;cursor:pointer}::ng-deep .amt-overlay{background-color:transparent}\n"] }]
        }], ctorParameters: function () { return []; } });

class AngularMaterialTimepickerDirective {
    constructor(viewContainerRef, overlay) {
        this.viewContainerRef = viewContainerRef;
        this.overlay = overlay;
        this.notifier = new Subject();
    }
    ngOnDestroy() {
        if (this.componentRef) {
            this.componentRef.destroy();
            this.notifier.next();
            this.notifier.complete();
        }
    }
    onFocus() {
        this.loadComponent();
    }
    /** Pop ups an instance of AMT using CDK Overlay. Also it's attached to the origin input. */
    loadComponent() {
        if (!this.componentRef) {
            this.origin = this.viewContainerRef.element.nativeElement;
            const overlayRef = this.overlay.create({
                positionStrategy: this.overlay
                    .position()
                    .flexibleConnectedTo(this.origin)
                    .withPositions([{
                        originX: 'start',
                        originY: 'bottom',
                        overlayX: 'start',
                        overlayY: 'top',
                    }]),
                scrollStrategy: this.overlay.scrollStrategies.block(),
                hasBackdrop: true,
                backdropClass: "amt-overlay",
            });
            const amtPortal = new ComponentPortal(AngularMaterialTimepickerComponent);
            this.componentRef = overlayRef.attach(amtPortal);
            // Trigger animations.
            this.componentRef.instance._startAnimation();
            // Close
            overlayRef.backdropClick()
                .pipe(take(1), tap(() => {
                this.notifier.next();
                this.componentRef?.instance._resetAnimation(); // Fires the animation;
            }), concatMap(() => this.componentRef.instance._animationDone))
                .subscribe(() => {
                overlayRef.dispose();
                this.componentRef = undefined; // if we don't do that, it will still pointing to destroyed instance.
            });
            this.setTimeOnPickerIfNeeded();
            this.setupListeners();
        }
    }
    /** Retrieves a valid hour/minutes time format from the origin input. If the value is not valid, returns an empty array. */
    getOriginInputHourMinutes() {
        const _default = this.origin?.value;
        if (_default && _default.match("^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$")) {
            return _default.split(":");
        }
        return [];
    }
    /** Sets the origin input time to the component instance if needed and fires change detection */
    setTimeOnPickerIfNeeded() {
        const currentOriginTime = this.getOriginInputHourMinutes();
        if (currentOriginTime.length > 0) {
            this.componentRef.instance.hour = currentOriginTime[0];
            this.componentRef.instance.minute = currentOriginTime[1];
            this.componentRef.changeDetectorRef.detectChanges();
        }
    }
    /**
     * Here we merged both emiters (hour and minutes) with combineLatest in order to generate a valid hour format for the input.
     */
    setupListeners() {
        const _instance = this.componentRef.instance;
        const currentOriginTime = this.getOriginInputHourMinutes();
        combineLatest([
            _instance.hourClicked
                .pipe(startWith(currentOriginTime?.[0] || _instance.hour)),
            _instance.minuteClicked
                .pipe(startWith(currentOriginTime?.[1] || _instance.minute))
        ])
            .pipe(takeUntil(this.notifier), map(batch => `${batch[0]}:${batch[1]}`))
            .subscribe(res => {
            this.origin.value = res;
        });
    }
}
AngularMaterialTimepickerDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerDirective, deps: [{ token: i0.ViewContainerRef }, { token: i1$1.Overlay }], target: i0.ɵɵFactoryTarget.Directive });
AngularMaterialTimepickerDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.9", type: AngularMaterialTimepickerDirective, selector: "input[angularMaterialTimepicker]", host: { listeners: { "focus": "onFocus()" } }, providers: [AngularMaterialTimepickerComponent], ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'input[angularMaterialTimepicker]',
                    providers: [AngularMaterialTimepickerComponent]
                }]
        }], ctorParameters: function () { return [{ type: i0.ViewContainerRef }, { type: i1$1.Overlay }]; }, propDecorators: { onFocus: [{
                type: HostListener,
                args: ['focus']
            }] } });

class MaterialModule {
}
MaterialModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: MaterialModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
MaterialModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: MaterialModule, imports: [CommonModule,
        MatRippleModule,
        OverlayModule], exports: [MatRippleModule,
        OverlayModule] });
MaterialModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: MaterialModule, imports: [[
            CommonModule,
            MatRippleModule,
            OverlayModule
        ], MatRippleModule,
        OverlayModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: MaterialModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [
                        CommonModule,
                        MatRippleModule,
                        OverlayModule
                    ],
                    exports: [
                        MatRippleModule,
                        OverlayModule
                    ]
                }]
        }] });

class TestAMTDirectiveComponent {
}
TestAMTDirectiveComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: TestAMTDirectiveComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
TestAMTDirectiveComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.9", type: TestAMTDirectiveComponent, selector: "ng-component", ngImport: i0, template: `<input type="text" angularMaterialTimepicker>`, isInline: true, directives: [{ type: AngularMaterialTimepickerDirective, selector: "input[angularMaterialTimepicker]" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: TestAMTDirectiveComponent, decorators: [{
            type: Component,
            args: [{
                    template: `<input type="text" angularMaterialTimepicker>`
                }]
        }] });

class AngularMaterialTimepickerModule {
}
AngularMaterialTimepickerModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
AngularMaterialTimepickerModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerModule, declarations: [AngularMaterialTimepickerComponent,
        HourFormatPipe,
        MinuteFormatPipe,
        AngularMaterialTimepickerDirective,
        TestAMTDirectiveComponent], imports: [MaterialModule,
        CommonModule], exports: [AngularMaterialTimepickerDirective] });
AngularMaterialTimepickerModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerModule, imports: [[
            MaterialModule,
            CommonModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AngularMaterialTimepickerModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        AngularMaterialTimepickerComponent,
                        HourFormatPipe,
                        MinuteFormatPipe,
                        AngularMaterialTimepickerDirective,
                        TestAMTDirectiveComponent,
                    ],
                    imports: [
                        MaterialModule,
                        CommonModule
                    ],
                    exports: [
                        AngularMaterialTimepickerDirective
                    ]
                }]
        }] });

/*
 * Public API Surface of angular-material-timepicker
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AngularMaterialTimepickerComponent, AngularMaterialTimepickerDirective, AngularMaterialTimepickerModule };
//# sourceMappingURL=jfilgaira-angular-material-timepicker.mjs.map
